<?php
/**
 * @package     CSVI
 * @subpackage  JoomlaContent
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace content\com_content\model\import;

defined('_JEXEC') or die;

use Exception;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Utilities\ArrayHelper;

/**
 * Content import.
 *
 * @package     CSVI
 * @subpackage  JoomlaContent
 * @since       6.0
 */
class Content extends \RantaiImportEngine
{
	/**
	 * The Joomla content helper
	 *
	 * @var    \Com_ContentHelperCom_Content
	 * @since  6.0
	 */
	protected $helper = null;
	/**
	 * Content table.
	 *
	 * @var    \ContentTableContent
	 * @since  6.0
	 */
	private $content = null;
	/**
	 * List of available custom fields
	 *
	 * @var    array
	 * @since  7.2.0
	 */
	private $customFields = '';

	/**
	 * @var \RantaiPluginDispatcher
	 * @since 9.0.0
	 */
	private $dispatcher;

	/**
	 * Run this before we start.
	 *
	 * @return  void.
	 *
	 * @since   7.2.0
	 * @throws  Exception
	 *
	 */
	public function onBeforeStart()
	{
		// Load the dispatcher
		$this->dispatcher = new \RantaiPluginDispatcher;
		$this->dispatcher->importPlugins('csviext', $this->db);

		// Load the tables that will contain the data
		$this->loadCustomFields();
	}

	/**
	 * Get a list of custom fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   7.2.0
	 *
	 * @throws  Exception
	 */
	private function loadCustomFields()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('name'))
			->from($this->db->quoteName('#__fields'))
			->where($this->db->quoteName('state') . ' = 1')
			->where($this->db->quoteName('context') . ' = ' . $this->db->quote('com_content.article'));
		$this->db->setQuery($query);
		$this->customFields = $this->db->loadObjectList();

		$this->log->add('Load the Joomla custom fields for articles');
	}

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.0
	 */
	public function getStart()
	{
		$data = $this->fields->getData();

		if (empty($data) || !$this->fields->getProcessRecord())
		{
			$this->loaded = false;

			return true;
		}

		foreach ($data as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'category_path':
						$this->setState('catid', $this->helper->getCategoryId($value, ''));
						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$this->loaded = true;

		// Check if there are required fields and then proceed
		$neededFields       = ['title~alias', 'catid~category_path'];
		$checkMissingFields = $this->fields->checkRequiredFields($neededFields);

		if ($checkMissingFields && !$this->getState('id', false))
		{
			$field = ['id or alias or title', 'catid or category_path'];
			$this->log->addStats('skipped', Text::sprintf('COM_CSVI_NO_REQUIRED_FIELD_FOUND', implode(',', $field)));
			$this->loaded = false;
		}

		if ($this->loaded)
		{
			// If there is title and no alias generate alias
			if ($this->getState('title', false) && !$this->getState('alias', false))
			{
				$this->generateAlias($this->getState('title', false));
			}

			// Get content id if available
			$contentId = $this->helper->getContentId($this->getState('alias', false), $this->getState('catid', false));

			if ($this->getState('id', false))
			{
				$contentId = $this->getState('id', false);
			}

			if ($contentId)
			{
				if ($this->template->get('delete_content_images', false))
				{
					$query = $this->db->getQuery(true)
						->update($this->db->quoteName('#__content'))
						->set($this->db->quoteName('images') . ' = ' . $this->db->quote('{}'))
						->where($this->db->quoteName('id') . ' = ' . (int) $contentId);
					$this->db->setQuery($query)->execute();
					$this->log->add('Delete existing images');
				}

				$this->content->load($contentId);
				$this->setState('id', $contentId);
				$title = $this->getState('title', false);

				if (!$title)
				{
					$title = $this->content->title;
				}
			}
			else
			{
				$this->setState('id', 0);
				$title = $this->getState('title', $this->getState('alias', false));
			}

			$this->setState('title', $title);

			// Inform user that he has no introtext and fulltext
			if (!$this->getState('id', false) && trim($this->getState('introtext', false)) === ''
				&& trim(
					$this->getState('fulltext', false)
				) === '')
			{
				$this->log->addStats('information', \Text::_('COM_CSVI_ARTICLE_HAS_NO_TEXT'));
			}

			$this->content->bind(ArrayHelper::fromObject($this->state));

			if (!$this->content->check())
			{
				$this->loaded = false;
			}

			if ($this->getState('id', false) && $this->content->load($this->getState('id', false)))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->add(
						'Article ' . $this->getState('alias')
						. 'not updated because the option overwrite existing data is set to No'
					);
					$this->loaded = false;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats(
					'skipped', Text::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('alias', ''))
				);
			}
			else
			{
				$this->setAttributes();
				$this->setImages();
				$this->setUrls();
				$this->setMetadata();
				$data = ArrayHelper::fromObject($this->state);

				// Add a creating date if there is no product_id
				if (!$this->getState('id', false))
				{
					if (!$this->getState('created_by'))
					{
						$this->content->created_by = $this->userId;
					}

					if (!$this->getState('created'))
					{
						$this->content->created = $this->date->toSql();
					}

					if (!$this->getState('metadesc', false))
					{
						$this->content->metadesc = '';
					}

					if (!$this->getState('modified', false))
					{
						$this->content->modified = $this->date->toSql();
					}

					if (!$this->getState('introtext', false))
					{
						$this->content->introtext = '';
					}
				}
				else
				{
					if (!$this->getState('modified_by'))
					{
						$this->content->modified_by = $this->userId;
					}
				}

				if (!$this->getState('modified', false))
				{
					$this->content->modified = $this->date->toSql();
				}

				$this->content->bind($data);

				// Check if we use a given order id
				if ($this->template->get('keepid'))
				{
					$this->content->checkId();
				}

				try
				{
					$result = $this->content->store();

					if (!$result)
					{
						return $result;
					}

					if (!$this->getState('id', 0))
					{
						$this->log->addStats('Added', Text::_('COM_CSVI_JOOMLA_CONTENT_ADDED'));
					}
					else
					{
						$this->log->addStats('Updated', Text::_('COM_CSVI_JOOMLA_CONTENT_UPDATED'));
					}

					$this->processCustomFields($this->content->id);
					$this->processTags($this->content->id);

					// Check if there is associations to import
					if ($this->getState('associations', false))
					{
						$this->processAssociations($this->content->id);
					}

					$this->storeWorkflow($this->content->id);

					if ($rules = $this->getState('rules_raw'))
					{
						$this->csvihelper->setRules($this->content->asset_id, $rules);
					}
				}
				catch (Exception $e)
				{
					$this->log->add('Cannot add Joomla content. Error: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', $e->getMessage());

					return false;
				}

				return true;
			}
		}

		return false;
	}

	/**
	 * Set the attributes field.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	private function setAttributes()
	{
		// Check for attributes
		if (!$this->getState('attribs', false))
		{
			$attributeFields =
				[
					'show_title',
					'link_titles',
					'show_intro',
					'show_category',
					'link_category',
					'show_parent_category',
					'link_parent_category',
					'show_author',
					'link_author',
					'show_create_date',
					'show_modify_date',
					'show_publish_date',
					'show_item_navigation',
					'show_icons',
					'show_print_icon',
					'show_email_icon',
					'show_vote',
					'show_hits',
					'show_noauth',
					'urls_position',
					'alternative_readmore',
					'article_layout',
					'show_publishing_options',
					'show_article_options',
					'show_urls_images_backend',
					'show_urls_images_frontend',
					'article_page_title',
					'show_tags',
					'info_block_position',
					'info_block_show_title',
					'show_associations',
				];

			// Get Value from content plugin
			$dispatcher = new \RantaiPluginDispatcher;
			$dispatcher->importPlugins('csviext', $this->db);

			// Fire the plugin to get attributes to import
			$pluginFields = $dispatcher->trigger(
				'getAttributes',
				[
					'extension' => 'joomla',
					'operation' => 'content',
					'log'       => $this->log,
				]
			);

			if (!empty($pluginFields[0]))
			{
				$this->log->add('Attributes added for content swmap plugin', false);
				$attributeFields = array_merge($attributeFields, $pluginFields[0]);
			}

			// Load the current attributes
			$attributes = json_decode($this->content->attribs);

			if (!is_object($attributes))
			{
				$attributes = new \stdClass;
			}

			foreach ($attributeFields as $field)
			{
				if ($this->getState($field, false))
				{
					$attributes->$field = $this->getState($field, '');
				}
			}

			// Store the new attributes
			$this->setState('attribs', json_encode($attributes));
		}
	}

	/**
	 * Set the images.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	private function setImages()
	{
		// Image handling
		$imageHelper = new \CsviHelperImage($this->template, $this->log, $this->csvihelper);

		if (!$this->getState('images'))
		{
			$imageFields =
				[
					'image_intro',
					'float_intro',
					'image_intro_alt',
					'image_intro_caption',
					'image_fulltext',
					'float_fulltext',
					'image_fulltext_alt',
					'image_fulltext_caption',
				];

			// Load the current images
			$images = json_decode($this->content->images ?? '{}');

			if (!is_object($images))
			{
				$images = new \stdClass;
			}

			foreach ($imageFields as $field)
			{
				if ($field === 'image_intro' || $field === 'image_fulltext')
				{
					// Image handling
					$imgPath = $this->template->get('file_location_image_files', 'images/');

					// Make sure the final slash is present
					if (substr($imgPath, -1) !== '/')
					{
						$imgPath .= '/';
					}

					$imagesContent = [];

					if ($this->getState($field, false))
					{
						$imagesContent[$field] = $this->getState($field, '');
					}

					foreach ($imagesContent as $keyField => $image)
					{
						$fileDetails = [];

						if ($imageHelper->isRemote($image))
						{
							$original = $image;
							$fullPath = $imgPath;
						}
						else
						{
							// Check if the image contains the image path
							$dirname = dirname($image);

							if (strpos($imgPath, $dirname) !== false)
							{
								// Collect rest of folder path if it is more than image default path
								$imageLeftPath = str_replace($imgPath, '', $dirname . '/');
								$image         = basename($image);

								if ($imageLeftPath)
								{
									$image = $imageLeftPath . $image;
								}
							}

							$original = $imgPath . $image;

							// Get subfolders
							$pathParts = pathinfo($original);
							$fullPath  = $pathParts['dirname'] . '/';
						}

						if ($this->template->get('process_image', false))
						{
							$fileDetails = $imageHelper->processImage($original, $fullPath);
						}
						else
						{
							$fileDetails['exists']      = true;
							$fileDetails['isimage']     = $imageHelper->isImage(JPATH_SITE . '/' . $original);
							$fileDetails['name']        = $image;
							$fileDetails['output_name'] = basename($image);
							$fileDetails['output_path'] = $fullPath;
						}

						if ($fileDetails['exists'])
						{
							$processedImage = (empty($fileDetails['output_path'])) ? $fileDetails['output_name']
								: $fileDetails['output_path'] . $fileDetails['output_name'];
							$this->setState($keyField, $processedImage);
						}
					}
				}

				if ($this->getState($field, false))
				{
					$images->$field = $this->getState($field, '');
				}
			}

			// Store the new attributes
			$this->setState('images', json_encode($images));
		}
	}

	/**
	 * Set the urls.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	private function setUrls()
	{
		if (!$this->getState('urls'))
		{
			$urlFields =
				[
					'urla',
					'urlatext',
					'targeta',
					'urlb',
					'urlbtext',
					'targetb',
					'urlc',
					'urlctext',
					'targetc',
				];

			// Load the current images
			$urls = json_decode($this->content->urls);

			if (!is_object($urls))
			{
				$urls = new \stdClass;
			}

			foreach ($urlFields as $field)
			{
				if ($this->getState($field, false))
				{
					$urls->$field = $this->getState($field, '');
				}
			}

			// Store the new attributes
			$this->setState('urls', json_encode($urls));
		}
	}

	/**
	 * Set the meta data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	private function setMetadata()
	{
		if (!$this->getState('metadata', false))
		{
			$metadataFields =
				[
					'meta_robots',
					'meta_author',
					'meta_rights',
					'meta_xreference',
				];

			// Load the current attributes
			$metadata = json_decode($this->content->metadata);

			if (!is_object($metadata))
			{
				$metadata = new \stdClass;
			}

			foreach ($metadataFields as $field)
			{
				$newField = str_ireplace('meta_', '', $field);

				if ($this->getState($field, false))
				{
					if ($this->getState($field, '') == '*')
					{
						$metadata->$field = '';
					}
					else
					{
						$metadata->$newField = $this->getState($field, '');
					}
				}
				elseif (!isset($metadata->$newField))
				{
					$metadata->$newField = '';
				}
			}

			// Store the new attributes
			$this->setState('metadata', json_encode($metadata));
		}
	}

	/**
	 * Update custom fields data.
	 *
	 * @param   int  $id  Id of the article
	 *
	 * @return  bool Returns true if all is OK | Returns false otherwise
	 *
	 * @since   7.2.0
	 */
	private function processCustomFields($id)
	{
		if (count($this->customFields) === 0)
		{
			$this->log->add('No custom fields found', false);

			return false;
		}

		$importFields       = $this->fields->getFields();
		$importCustomFields = [];

		foreach ($importFields as $importField)
		{
			foreach ($importField as $field)
			{
				$rawFieldName         = $field->field_name;
				$importCustomFields[] = $rawFieldName;

				if ($rawFieldName === 'rules_raw')
				{
					continue;
				}

				if (strpos($rawFieldName, '_raw') !== false)
				{
					$stdclass             = new \stdClass();
					$stdclass->name       = $rawFieldName;
					$this->customFields[] = $stdclass;
				}
			}
		}

		$skipEmptyFields = $this->template->get('skipEmptyFields', false);

		foreach ($this->customFields as $field)
		{
			$fieldName = $field->name;

			if (($skipEmptyFields && $this->getState($fieldName, '') === '')
				|| !in_array($fieldName, $importCustomFields))
			{
				continue;
			}

			$this->dispatcher->trigger(
				'importCustomfields',
				[
					'plugin'   => 'joomlacustomfields',
					'field'    => $field->name,
					'value'    => $this->getState($fieldName, ''),
					'item_id'  => $id,
					'log'      => $this->log,
					'template' => $this->template,
				]
			);
		}

		return true;
	}

	/**
	 * Update Tags data
	 *
	 * @param   int  $id  Id of the article
	 *
	 * @return  bool Returns true if all is OK | Returns false otherwise
	 *
	 * @since   7.7.0
	 */
	private function processTags($id)
	{
		$tags = $this->getState('tags', false);

		if (!$tags)
		{
			return false;
		}

		$appendTags = $this->template->get('append_tags', false);

		if (!$appendTags)
		{
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__contentitem_tag_map'))
				->where($this->db->quoteName('content_item_id') . ' = ' . (int) $id);
			$this->db->setQuery($query)->execute();
			$this->log->add('Delete existing tags to insert new ones');
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('core_content_id'))
			->from($this->db->quoteName('#__ucm_content'))
			->where($this->db->quoteName('core_alias') . '  = ' . $this->db->quote($this->getState('alias')));
		$this->db->setQuery($query);
		$ucmId = $this->db->loadResult();

		if ($ucmId)
		{
			$query->clear()
				->delete($this->db->quoteName('#__ucm_content'))
				->where($this->db->quoteName('core_content_id') . ' = ' . (int) $ucmId);
			$this->db->setQuery($query)->execute();

			$query->clear()
				->delete($this->db->quoteName('#__ucm_base'))
				->where($this->db->quoteName('ucm_id') . ' = ' . (int) $ucmId);
			$this->db->setQuery($query)->execute();
		}

		$content = $this->getState('fulltext');

		if (!$content && $this->getState('introtext'))
		{
			$content = $this->getState('introtext');
		}

		$query->clear()
			->insert($this->db->quoteName('#__ucm_content'))
			->columns(
				$this->db->quoteName(
					[
						'core_type_alias',
						'core_title',
						'core_alias',
						'core_body',
						'core_state',
						'core_access',
						'core_content_item_id',
						'core_type_id',
						'core_catid',
						'core_created_time',
						'core_modified_time',
						'core_params',
						'core_metadata',
						'core_images',
						'core_urls',
						'core_publish_up',
						'core_publish_down',
					]
				)
			)
			->values(
				$this->db->quote('com_content.article') . ', '
				. $this->db->quote($this->getState('title')) . ', '
				. $this->db->quote($this->getState('alias')) . ', '
				. $this->db->quote($content) . ', 1, 1, ' .
				(int) $this->getState('id') . ', 1, '
				. (int) $this->getState('catid') . ','
				. $this->db->quote($this->date->toSql()) . ','
				. $this->db->quote($this->date->toSql()) . ','
				. $this->db->quote($this->getState('attribs')) . ','
				. $this->db->quote($this->getState('metadata')) . ','
				. $this->db->quote($this->getState('images')) . ','
				. $this->db->quote($this->getState('urls')) . ','
				. $this->db->quote($this->getState('publish_up', $this->db->getNullDate())) . ','
				. $this->db->quote($this->getState('publish_down', $this->db->getNullDate()))
			);
		$this->db->setQuery($query)->execute();
		$ucmId = $this->db->insertid();

		$tagsArray = explode('|', $tags);

		$typeAlias = 'com_content.article';
		$query->clear()
			->select($this->db->quoteName('type_id'))
			->from($this->db->quoteName('#__content_types'))
			->where($this->db->quoteName('type_alias') . '  = ' . $this->db->quote($typeAlias));
		$this->db->setQuery($query);
		$typeId  = $this->db->loadResult();
		$tagDate = $this->date->toSql();

		foreach ($tagsArray as $tag)
		{
			$query->clear()
				->select($this->db->quoteName('id'))
				->from($this->db->quoteName('#__tags'))
				->where($this->db->quoteName('path') . '  = ' . $this->db->quote($tag));
			$this->db->setQuery($query);
			$tagId = $this->db->loadResult();
			$this->log->add('Get the tag id ');

			if (!$tagId)
			{
				$this->log->add('No tag id found for the tag ' . $tag, false);
				continue;
			}

			// Delete the values and do a fresh import to avoid dulicate error
			$query->clear()
				->delete($this->db->quoteName('#__contentitem_tag_map'))
				->where($this->db->quoteName('content_item_id') . ' = ' . (int) $id)
				->where($this->db->quoteName('tag_id') . ' = ' . (int) $tagId);
			$this->db->setQuery($query)->execute();
			$this->log->add('Removed existing tag for content before inserting');

			$query->clear()
				->insert($this->db->quoteName('#__contentitem_tag_map'))
				->columns(
					$this->db->quoteName(
						['type_alias', 'content_item_id', 'tag_id', 'tag_date', 'type_id', 'core_content_id',]
					)
				)
				->values(
					$this->db->quote($typeAlias) . ', ' . (int) $id . ', ' .
					(int) $tagId . ', ' . $this->db->quote($tagDate) . ', ' . (int) $typeId . ', ' . (int) $ucmId
				);
			$this->db->setQuery($query)->execute();
			$this->log->add('Insert the new tag for content');
		}

		// Do insert to #__ucm_base table
		$query->clear()
			->insert($this->db->quoteName('#__ucm_base'))
			->columns(
				$this->db->quoteName(
					[
						'ucm_id',
						'ucm_item_id',
						'ucm_type_id',
						'ucm_language_id',
					]
				)
			)
			->values(
				(int) $ucmId . ','
				. (int) $id . ','
				. (int) $typeId
				. ',0'
			);
		$this->db->setQuery($query)->execute();

		return true;
	}

	/**
	 * Process associated content
	 *
	 * @param   int  $contentId  Id of the content
	 *
	 * @return  bool True if all ok False otherwise.
	 *
	 * @since   7.10.0
	 */
	private function processAssociations($contentId)
	{
		if ($this->getState('language', false) === '*')
		{
			$this->log->add('Association cannot be added if language is set to all', false);
			$this->log->addStats('incorrect', 'COM_CSVI_LANGUAGE_SET_TO_ALL');

			return false;
		}

		if ($this->getState('associations_category_path', false))
		{
			$associatedCategories      = explode('|', $this->getState('associations_category_path', false));
			$associatedCategoryIdValue = [];

			foreach ($associatedCategories as $associatedKey => $associatedCategory)
			{
				[$associatedLanguage, $associatedPath] = explode('#', $associatedCategory);
				$associatedCategoryId = $this->helper->getCategoryId($associatedPath, $associatedLanguage);
				$convertCategoryIds   = [];

				if ($associatedCategoryId)
				{
					$convertCategoryIds[]        = $associatedLanguage;
					$convertCategoryIds[]        = $associatedCategoryId;
					$associatedCategoryIdValue[] = implode('#', $convertCategoryIds);
				}
			}

			$newCategoryIdsValues = implode('|', $associatedCategoryIdValue);
			$this->setState('associations_category_id', $newCategoryIdsValues);
		}

		$languageCategoryArray = [];

		if ($this->getState('associations_category_id', false))
		{
			$associatedCategoryIds = explode('|', $this->getState('associations_category_id', false));

			foreach ($associatedCategoryIds as $associatedCategoryId)
			{
				[$associatedLanguage, $associatedId] = explode('#', $associatedCategoryId);

				$languageCategoryArray[$associatedLanguage] = $associatedId;
			}
		}

		$associations                   = [];
		$associatedContents             = explode('|', $this->getState('associations', false));
		$associationsContext            = 'com_content.item';
		$primaryLanguage                = $this->getState('language', false);
		$associations[$primaryLanguage] = $contentId;

		foreach ($associatedContents as $content)
		{
			$explodeAssociatedContent = explode('#', $content);
			$associatedLanguage       = $explodeAssociatedContent[0];
			$associatedAlias          = $explodeAssociatedContent[1];
			$associatedCatId          = $this->helper->getAssociationCategory($associatedAlias, $associatedLanguage);

			if (isset($explodeAssociatedContent[2]))
			{
				$associatedCatId = $explodeAssociatedContent[2];
			}

			if (strpos($associatedLanguage, '-') === false)
			{
				$this->log->add('Not a valid language ' . $associatedLanguage, false);
				$this->log->addStats('incorrect', \Text::sprintf('COM_CSVI_LANGUAGE_NOT_VALID', $associatedLanguage));
				continue;
			}

			if (!$associatedCatId)
			{
				$associatedCatId = $languageCategoryArray[$associatedLanguage] ?? $this->getState('catid', false);
			}

			$languageTags        = explode('-', $associatedLanguage);
			$languageTags[1]     = strtoupper($languageTags[1]);
			$language            = implode('-', $languageTags);
			$associatedContentId = $this->helper->getContentId($associatedAlias, $associatedCatId);

			if ($associatedContentId)
			{
				$associations[$language] = (int) $associatedContentId;
			}
			else
			{
				$this->log->add('No associated content id found with alias  ' . $associatedAlias, false);
				$this->log->addStats('incorrect', \Text::sprintf('COM_CSVI_NO_ARTICLE_FOUND', $associatedAlias));
			}
		}

		// Make sure there are no duplicates
		$associations = array_unique($associations);

		if ((count($associations)) > 1)
		{
			$key   = md5(json_encode($associations));
			$query = $this->db->getQuery(true)
				->insert($this->db->quoteName('#__associations'));

			foreach ($associations as $arrayVal => $id)
			{
				$this->deleteAssociation($id, $associationsContext);
				$query->values(
					((int) $id) . ',' . $this->db->quote($associationsContext) . ',' . $this->db->quote($key)
				);
			}

			$this->db->setQuery($query)->execute();
			$this->log->add('Associations added for content');
		}

		return true;
	}

	/**
	 * Delete a association
	 *
	 * @param   int  $id       Id of the content
	 * @param   int  $context  Context of com_content
	 *
	 * @return  boolean True if deleted | False if id is missing.
	 *
	 * @since   7.10.0
	 */
	private function deleteAssociation($id, $context)
	{
		if (!$id)
		{
			$this->log->add('Association id not found to delete', false);

			return false;
		}

		$query = $this->db->getquery(true)
			->delete($this->db->quotename('#__associations'))
			->where($this->db->quotename('id') . ' = ' . (int) $id)
			->where($this->db->quotename('context') . ' = ' . $this->db->quote($context));
		$this->db->setquery($query)->execute();
		$this->log->add('Association content deleted');

		return true;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->content = $this->getTable('Content');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->content->reset();
	}

	/**
	 * Generate alias from title
	 *
	 * @param   string  $title  Title of the article
	 *
	 * @return  void
	 *
	 * @since   7.16.1
	 */
	private function generateAlias($title)
	{
		$translit = new \CsviHelperTranslit($this->template);
		$alias    = $translit->stringURLSafe($title);

		if (!$alias)
		{
			$alias = Factory::getDate()->format('Y-m-d-H-i-s');
		}

		$this->setState('alias', $alias);

		if ($this->template->get('recreate_alias', false))
		{
			$this->recreateAlias($title);
		}
	}

	/**
	 * Create new alias if user wants to.
	 *
	 * @param   string  $title  Title of the article
	 *
	 * @return  void.
	 *
	 * @since   8.0.0
	 */
	private function recreateAlias(string $title): void
	{
		$alias = $this->getState('alias');
		$query = $this->db->getQuery(true)
			->select('COUNT(' . $this->db->quoteName('id') . ')')
			->from($this->db->quoteName('#__content'))
			->where($this->db->quoteName('alias') . ' = ' . $this->db->quote($alias))
			->where($this->db->quoteName('id') . ' != ' . (int) $this->getState('id'));
		$this->db->setQuery($query);
		$id = $this->db->loadResult();
		$this->log->add('Check if the alias exists');

		if ($id > 0)
		{
			if ($title)
			{
				$query->clear()
					->select(
						'ABS(REPLACE(' . $this->db->quoteName('alias') . ', ' . $this->db->quote($alias) . ', '
						. $this->db->quote('') . ')) + 1'
					)
					->from($this->db->quoteName('#__content'))
					->where(
						'('
						. $this->db->quoteName('alias') . ' = ' . $this->db->quote($alias)
						. ' OR '
						. $this->db->quoteName('alias') . ' LIKE ' . $this->db->quote($alias . '-_%')
						. ' AND RIGHT(' . $this->db->quoteName('alias') . ', 1) IN (0,1,2,3,4,5,6,7,8,9)'
						. ')'
					)
					->where($this->db->quoteName('id') . ' != ' . (int) $this->getState('id'))
					->order('LENGTH(' . $this->db->quoteName('alias') . '), ' . $this->db->quoteName('alias') . ' ASC');
				$this->db->setQuery($query);
				$items = $this->db->loadColumn();
				$this->log->add('Check how many other articles exist with the same alias');

				if ($items)
				{
					$count = array_pop($items);
					$alias .= '-' . $count;
				}
			}
			else
			{
				$jdate = Factory::getDate();
				$alias .= $jdate->format("Y-m-d-h-i-s") . mt_rand();
			}
		}

		$this->setState('alias', $alias);
	}

	/**
	 * Store workflow in Joomla4
	 *
	 * @param   int  $contentId  Id of the content
	 *
	 * @return  void
	 *
	 * @since   8.0.0
	 */
	private function storeWorkflow(int $contentId): void
	{
		$tableList = $this->db->getTableList();
		$tableName = $this->db->getPrefix() . 'workflow_associations';

		if (!in_array($tableName, $tableList, true))
		{
			return;
		}

		$query = $this->db->getquery(true)
			->delete($this->db->quotename('#__workflow_associations'))
			->where($this->db->quotename('item_id') . ' = ' . $contentId)
			->where($this->db->quotename('extension') . ' = ' . $this->db->quote('com_content.article'));
		$this->db->setquery($query)->execute();
		$this->log->add('Workflow content deleted');

		$query->clear()
			->insert($this->db->quoteName('#__workflow_associations'))
			->columns($this->db->quoteName(['item_id', 'stage_id', 'extension']))
			->values($contentId . ',1, ' . $this->db->quote('com_content.article'));
		$this->db->setQuery($query)->execute();
		$this->log->add('Workflow content added');
	}
}
